.theme-tab {
  &.v-tabs {
    .v-tab {
      border-radius: $border-radius-root !important;
      min-width: auto !important;
      &.v-slide-group-item--active {
        background: rgb(var(--v-theme-primary));
      }
    }
  }
}
